@@ -154,7 +154,7 @@ pub fn to_extension_map(config: &HashMap<String, Vec<String>>) -> HashMap<String
     let mut new_map = HashMap::new();
     for (command, extensions) in config.iter() {
         for ext in extensions.iter() {
-            new_map.insert(ext.clone(), command.clone());
+            new_map.insert(ext.to_lowercase(), command.clone());
         }
     }
     new_map
@@ -202,27 +202,3 @@ pub fn get_contents_r(path: PathBuf, vec: &mut Vec<PathBuf>) -> Result<Vec<PathB
     }
     Ok(vec.clone())
 }
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    #[test]
-    fn get_contents_r_test() {
-        let path = PathBuf::from("/home/kyohei/nim/new");
-        let mut vec = vec![];
-        let result = get_contents_r(path.clone(), &mut vec).unwrap();
-        let mut bt = BTreeSet::new();
-        bt.insert(path.clone());
-        for p in result {
-            bt.insert(p);
-        }
-
-        let mut target = BTreeSet::new();
-        for entry in walkdir::WalkDir::new(path) {
-            let entry = entry.unwrap();
-            target.insert(entry.path().to_path_buf());
-        }
-        assert_eq!(bt, target);
-    }
-}
