@@ -7,7 +7,6 @@ use crate::session::*;
 use std::ffi::OsStr;
 // use clipboard::{ClipboardContext, ClipboardProvider};
 use log::debug;
-use log::error;
 use std::io::{stdin, stdout, Write};
 use std::path::{Path, PathBuf};
 use std::time::Instant;
@@ -19,45 +18,26 @@ use termion::{clear, cursor, screen};
 
 pub fn run(arg: PathBuf) -> Result<(), MyError> {
     env_logger::init();
-    debug!("starts initial setup.");
+    debug!("Initial setup starts.");
 
-    let mut config_dir = dirs::config_dir().unwrap_or_else(|| panic!("Cannot read config dir."));
-    config_dir.push(FX_CONFIG_DIR);
-    let config_file = config_dir.join(PathBuf::from(CONFIG_FILE));
-    let trash_dir = config_dir.join(PathBuf::from(TRASH));
-    make_config(&config_file, &trash_dir)
+    let mut config_dir_path =
+        dirs::config_dir().unwrap_or_else(|| panic!("Cannot read config dir."));
+    config_dir_path.push(FX_CONFIG_DIR);
+    let config_file_path = config_dir_path.join(PathBuf::from(CONFIG_FILE));
+    let trash_dir_path = config_dir_path.join(PathBuf::from(TRASH));
+    make_config(&config_file_path, &trash_dir_path)
         .unwrap_or_else(|_| panic!("Cannot make config file or trash dir."));
-    let session_file = config_dir.join(PathBuf::from(SESSION_FILE));
-    make_session(&session_file).unwrap_or_else(|_| panic!("Cannot make session file."));
+    let session_file_path = config_dir_path.join(PathBuf::from(SESSION_FILE));
+    make_session(&session_file_path).unwrap_or_else(|_| panic!("Cannot make session file."));
 
     if !&arg.exists() {
         println!("Invalid path: {}", &arg.display());
         return Ok(());
     }
 
-    let (column, row) = termion::terminal_size()?;
-    if column < 21 {
-        error!("too small terminal size.");
-        panic!("panic due to terminal size (less than 21 column).")
-    };
-
     let mut state = State::new();
-
-    let time_start = if column >= 49 { 31 } else { column - 17 };
-    let name_max: usize = if column >= 49 {
-        29
-    } else {
-        (time_start - 2).into()
-    };
-    state.layout = Layout {
-        terminal_row: row,
-        terminal_column: column,
-        name_max_len: name_max,
-        time_start_pos: time_start,
-    };
+    state.trash_dir = trash_dir_path;
     state.current_dir = arg.canonicalize()?;
-    state.update_list();
-    state.trash_dir = trash_dir;
 
     let mut filtered = false;
 
@@ -67,13 +47,14 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
 
     print!("{}", cursor::Hide);
 
+    state.update_list();
     clear_and_show(&state.current_dir);
     state.list_up(nums.skip);
 
     state.move_cursor(&nums, STARTING_POINT);
     screen.flush()?;
 
-    let mut p_memo_v: Vec<CursorMemo> = Vec::new();
+    let mut p_memo_v: Vec<ParentMemo> = Vec::new();
     let mut c_memo_v: Vec<ChildMemo> = Vec::new();
     let mut stdin = stdin().keys();
 
@@ -181,7 +162,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                 Key::Char('l') | Key::Char('\n') | Key::Right => {
                     if let Ok(item) = state.get_item(nums.index) {
                         match item.file_type {
-                            FileType::File | FileType::Symlink => {
+                            FileType::File => {
                                 print!("{}", screen::ToAlternateScreen);
                                 if let Err(e) = state.open_file(nums.index) {
                                     print_warning(e, y);
@@ -193,6 +174,56 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                 print!("{}", cursor::Hide);
                                 state.move_cursor(&nums, y);
                             }
+                            FileType::Symlink => match &item.symlink_dir_path {
+                                Some(true_path) => match std::fs::File::open(true_path) {
+                                    Err(e) => {
+                                        print_warning(e, y);
+                                        continue;
+                                    }
+                                    Ok(_) => {
+                                        let cursor_memo = if !filtered {
+                                            ParentMemo {
+                                                to_sym_dir: Some(state.current_dir.clone()),
+                                                num: nums.clone(),
+                                                cursor_pos: y,
+                                            }
+                                        } else {
+                                            ParentMemo {
+                                                to_sym_dir: Some(state.current_dir.clone()),
+                                                num: Num::new(),
+                                                cursor_pos: STARTING_POINT,
+                                            }
+                                        };
+                                        p_memo_v.push(cursor_memo);
+                                        filtered = false;
+
+                                        state.current_dir = true_path.clone();
+                                        if let Err(e) =
+                                            std::env::set_current_dir(&state.current_dir)
+                                        {
+                                            print_warning(e, y);
+                                            continue;
+                                        }
+                                        state.update_list();
+                                        clear_and_show(&state.current_dir);
+                                        nums.reset();
+                                        state.list_up(nums.skip);
+                                        state.move_cursor(&nums, STARTING_POINT);
+                                    }
+                                },
+                                None => {
+                                    print!("{}", screen::ToAlternateScreen);
+                                    if let Err(e) = state.open_file(nums.index) {
+                                        print_warning(e, y);
+                                        continue;
+                                    }
+                                    print!("{}", screen::ToAlternateScreen);
+                                    clear_and_show(&state.current_dir);
+                                    state.list_up(nums.skip);
+                                    print!("{}", cursor::Hide);
+                                    state.move_cursor(&nums, y);
+                                }
+                            },
                             FileType::Directory => {
                                 match std::fs::File::open(&item.file_path) {
                                     Err(e) => {
@@ -202,12 +233,14 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                     Ok(_) => {
                                         //store the last cursor position and skip number
                                         let cursor_memo = if !filtered {
-                                            CursorMemo {
+                                            ParentMemo {
+                                                to_sym_dir: None,
                                                 num: nums.clone(),
                                                 cursor_pos: y,
                                             }
                                         } else {
-                                            CursorMemo {
+                                            ParentMemo {
+                                                to_sym_dir: None,
                                                 num: Num::new(),
                                                 cursor_pos: STARTING_POINT,
                                             }
@@ -227,13 +260,10 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                         match c_memo_v.pop() {
                                             Some(memo) => {
                                                 if state.current_dir == memo.dir_path {
-                                                    nums = memo.cursor_memo.num;
+                                                    nums = memo.num;
                                                     clear_and_show(&state.current_dir);
                                                     state.list_up(nums.skip);
-                                                    state.move_cursor(
-                                                        &nums,
-                                                        memo.cursor_memo.cursor_pos,
-                                                    );
+                                                    state.move_cursor(&nums, memo.cursor_pos);
                                                 } else {
                                                     clear_and_show(&state.current_dir);
                                                     state.list_up(0);
@@ -264,68 +294,82 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                             let cursor_memo = if !filtered {
                                 ChildMemo {
                                     dir_path: pre.clone(),
-                                    cursor_memo: CursorMemo {
-                                        num: nums.clone(),
-                                        cursor_pos: y,
-                                    },
+                                    num: nums.clone(),
+                                    cursor_pos: y,
                                 }
                             } else {
                                 ChildMemo {
                                     dir_path: PathBuf::from(""),
-                                    cursor_memo: CursorMemo {
-                                        num: Num::new(),
-                                        cursor_pos: STARTING_POINT,
-                                    },
+                                    num: Num::new(),
+                                    cursor_pos: STARTING_POINT,
                                 }
                             };
                             c_memo_v.push(cursor_memo);
                             filtered = false;
 
-                            state.current_dir = parent_p.to_path_buf();
-                            std::env::set_current_dir(&state.current_dir)
-                                .unwrap_or_else(|e| print_warning(e, y));
-                            state.update_list();
-
                             match p_memo_v.pop() {
                                 Some(memo) => {
+                                    match memo.to_sym_dir {
+                                        Some(true_path) => {
+                                            state.current_dir = true_path;
+                                        }
+                                        None => {
+                                            state.current_dir = parent_p.to_path_buf();
+                                        }
+                                    }
+                                    if let Err(e) = std::env::set_current_dir(&state.current_dir) {
+                                        print_warning(e, y);
+                                        continue;
+                                    }
+                                    state.update_list();
                                     nums = memo.num;
                                     clear_and_show(&state.current_dir);
                                     state.list_up(nums.skip);
                                     state.move_cursor(&nums, memo.cursor_pos);
                                 }
-                                None => match pre.file_name() {
-                                    Some(name) => {
-                                        let mut new_pos = 0;
-                                        for (i, item) in state.list.iter().enumerate() {
-                                            let name_as_os_str: &OsStr = item.file_name.as_ref();
-                                            if name_as_os_str == name {
-                                                new_pos = i;
+                                None => {
+                                    state.current_dir = parent_p.to_path_buf();
+                                    if let Err(e) = std::env::set_current_dir(&state.current_dir) {
+                                        print_warning(e, y);
+                                        continue;
+                                    }
+                                    state.update_list();
+                                    match pre.file_name() {
+                                        Some(name) => {
+                                            let mut new_pos = 0;
+                                            for (i, item) in state.list.iter().enumerate() {
+                                                let name_as_os_str: &OsStr =
+                                                    item.file_name.as_ref();
+                                                if name_as_os_str == name {
+                                                    new_pos = i;
+                                                }
+                                            }
+                                            nums.index = new_pos;
+
+                                            if nums.index
+                                                >= (state.layout.terminal_row
+                                                    - (STARTING_POINT + 3))
+                                                    .into()
+                                            {
+                                                nums.skip = (nums.index - 3) as u16;
+                                                clear_and_show(&state.current_dir);
+                                                state.list_up(nums.skip);
+                                                state.move_cursor(&nums, STARTING_POINT + 3);
+                                            } else {
+                                                nums.skip = 0;
+                                                clear_and_show(&state.current_dir);
+                                                state.list_up(0);
+                                                state.move_cursor(&nums, (nums.index + 3) as u16);
                                             }
                                         }
-                                        nums.index = new_pos;
-
-                                        if nums.index
-                                            >= (state.layout.terminal_row - (STARTING_POINT + 3))
-                                                .into()
-                                        {
-                                            nums.skip = (nums.index - 3) as u16;
-                                            clear_and_show(&state.current_dir);
-                                            state.list_up(nums.skip);
-                                            state.move_cursor(&nums, STARTING_POINT + 3);
-                                        } else {
-                                            nums.skip = 0;
+                                        None => {
+                                            nums.reset();
                                             clear_and_show(&state.current_dir);
                                             state.list_up(0);
-                                            state.move_cursor(&nums, (nums.index + 3) as u16);
+                                            state.move_cursor(&nums, STARTING_POINT);
                                         }
                                     }
-                                    None => {
-                                        nums.reset();
-                                        clear_and_show(&state.current_dir);
-                                        state.list_up(0);
-                                        state.move_cursor(&nums, STARTING_POINT);
-                                    }
-                                },
+                                }
                             }
                         }
                         None => {
@@ -504,7 +548,7 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                     screen.flush()?;
 
                                     state.registered.clear();
-                                    let mut count = 0;
+                                    let mut count: usize = 0;
                                     let clone = state.list.clone();
                                     let iter = clone.iter().filter(|item| item.selected);
                                     let total_selected = iter.clone().count();
@@ -540,9 +584,18 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
                                     state.list_up(nums.skip);
 
                                     let duration = duration_to_string(start.elapsed());
-                                    let mut delete_message: String = count.to_string();
-                                    delete_message
-                                        .push_str(&format!(" items deleted [{}]", duration));
+                                    let delete_message: String = {
+                                        if count == 1 {
+                                            format!("1 item deleted [{}]", duration)
+                                        } else {
+                                            let mut count = count.to_string();
+                                            count.push_str(&format!(
+                                                " items deleted [{}]",
+                                                duration
+                                            ));
+                                            count
+                                        }
+                                    };
                                     print_info(delete_message, y);
                                     print!(" ");
 
@@ -1233,6 +1286,6 @@ pub fn run(arg: PathBuf) -> Result<(), MyError> {
     }
     //When exits, restore the cursor
     print!("{}", cursor::Restore);
-    state.write_session(session_file)?;
+    state.write_session(session_file_path)?;
     Ok(())
 }
