@@ -1,10 +1,9 @@
 use serde::{Deserialize, Serialize};
 use std::fs::read_to_string;
-
-use crate::state::FX_CONFIG_DIR;
+use super::errors::MyError;
+use super::state::FX_CONFIG_DIR;
 
 pub const SESSION_FILE: &str = ".session";
-
 pub const SESSION_EXAMPLE: &str = "sort_by = \"Name\"
 show_hidden = false
 ";
@@ -21,15 +20,15 @@ pub enum SortKey {
     Time,
 }
 
-pub fn read_session() -> Option<Session> {
-    let mut session = dirs::config_dir().unwrap_or_else(|| panic!("cannot read config dir."));
+pub fn read_session() -> Result<Session, MyError> {
+    let mut session = dirs::config_dir().unwrap_or_else(|| panic!("Cannot read config dir."));
     session.push(FX_CONFIG_DIR);
     session.push(SESSION_FILE);
     let session = read_to_string(session.as_path());
     if let Ok(session) = session {
-        let deserialized: Session = toml::from_str(&session).unwrap();
-        Some(deserialized)
+        let deserialized: Session = toml::from_str(&session)?;
+        Ok(deserialized)
     } else {
-        None
+        panic!("Cannot deserialize session file.");
     }
 }
\ No newline at end of file
