@@ -4,6 +4,7 @@ use super::functions::*;
 use super::nums::*;
 use super::session::*;
 use chrono::prelude::*;
+use log::error;
 use std::collections::HashMap;
 use std::collections::HashSet;
 use std::ffi::OsString;
@@ -15,6 +16,7 @@ use std::process::{Command, ExitStatus};
 use termion::{clear, color, cursor, style};
 
 pub const STARTING_POINT: u16 = 3;
+pub const TIME_WIDTH: u16 = 17;
 pub const DOWN_ARROW: char = '\u{21D3}';
 pub const RIGHT_ARROW: char = '\u{21D2}';
 pub const FX_CONFIG_DIR: &str = "felix";
@@ -23,7 +25,7 @@ pub const TRASH: &str = "trash";
 pub const WHEN_EMPTY: &str = "Are you sure to empty the trash directory? (if yes: y)";
 
 macro_rules! print_item {
-    ($color: expr, $name: expr, $time: expr, $selected: expr, $time_start_pos: expr, $column: expr) => {
+    ($color: expr, $name: expr, $time: expr, $selected: expr, $layout: expr) => {
         if *($selected) {
             print!(
                 "{}{}{}{}{}{} {}{}{}",
@@ -31,8 +33,8 @@ macro_rules! print_item {
                 style::Invert,
                 $name,
                 style::Reset,
-                cursor::Left(60),
-                cursor::Right($time_start_pos),
+                cursor::Left(100),
+                cursor::Right($layout.time_start_pos - 1),
                 style::Invert,
                 $time,
                 style::Reset
@@ -42,16 +44,19 @@ macro_rules! print_item {
                 "{}{}{}{} {}{}",
                 $color,
                 $name,
-                cursor::Left(60),
-                cursor::Right($time_start_pos),
+                cursor::Left(100),
+                cursor::Right($layout.time_start_pos - 1),
                 $time,
                 color::Fg(color::Reset)
             );
         }
-        if $column > 58 {
-            print!("{}", (0..8).map(|_| ' ').collect::<String>());
-        } else if $column > 49 {
-            print!("{}", (0..($column - 49)).map(|_| ' ').collect::<String>());
+        if $layout.terminal_column > $layout.time_start_pos + 16 {
+            print!(
+                "{}",
+                (0..($layout.terminal_column - $layout.time_start_pos - 16))
+                    .map(|_| ' ')
+                    .collect::<String>()
+            );
         }
     };
 }
@@ -67,6 +72,7 @@ pub struct State {
     pub sort_by: SortKey,
     pub layout: Layout,
     pub show_hidden: bool,
+    pub rust_log: Option<String>,
 }
 
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Clone)]
@@ -74,6 +80,7 @@ pub struct ItemInfo {
     pub file_type: FileType,
     pub file_name: String,
     pub file_path: std::path::PathBuf,
+    pub symlink_dir_path: Option<PathBuf>,
     pub file_size: u64,
     pub file_ext: Option<OsString>,
     pub modified: Option<String>,
@@ -93,12 +100,46 @@ pub struct Layout {
     pub terminal_column: u16,
     pub name_max_len: usize,
     pub time_start_pos: u16,
+    pub option_name_len: Option<usize>,
 }
 
 impl Default for State {
     fn default() -> Self {
         let config = read_config().unwrap();
         let session = read_session().unwrap();
+        let (column, row) = termion::terminal_size().unwrap();
+        if column < 21 {
+            error!("Too small terminal size.");
+            panic!("Panic due to terminal size (less than 21 columns).")
+        };
+        let mut time_start: u16;
+        let mut name_max: usize;
+        match config.item_name_length {
+            Some(option_max) => {
+                time_start = option_max as u16 + 3;
+                name_max = option_max;
+            }
+            None => {
+                time_start = if column >= 50 {
+                    33
+                } else {
+                    column - TIME_WIDTH
+                };
+                name_max = if column >= 50 {
+                    30
+                } else {
+                    (time_start - 3).into()
+                };
+            }
+        }
+
+        let required = time_start + TIME_WIDTH - 1;
+        if required > column {
+            let diff = required - column;
+            name_max -= diff as usize;
+            time_start -= diff;
+        }
+
         State {
             list: Vec::new(),
             registered: Vec::new(),
@@ -113,12 +154,14 @@ impl Default for State {
             commands: to_extension_map(&config.exec),
             sort_by: session.sort_by,
             layout: Layout {
-                terminal_row: 0,
-                terminal_column: 0,
-                name_max_len: 0,
-                time_start_pos: 0,
+                terminal_row: row,
+                terminal_column: column,
+                name_max_len: name_max,
+                time_start_pos: time_start,
+                option_name_len: config.item_name_length,
             },
             show_hidden: session.show_hidden,
+            rust_log: std::env::var("RUST_LOG").ok(),
         }
     }
 }
@@ -144,7 +187,7 @@ impl State {
 
         match extention {
             Some(extention) => {
-                let ext = extention.to_os_string().into_string().unwrap();
+                let ext = extention.to_ascii_lowercase().into_string().unwrap();
                 match map.get(&ext) {
                     Some(command) => {
                         let mut ex = Command::new(command);
@@ -419,7 +462,7 @@ impl State {
         let name = if chars.len() > self.layout.name_max_len {
             let mut result = chars
                 .iter()
-                .take(self.layout.name_max_len - 3)
+                .take(self.layout.name_max_len - 2)
                 .collect::<String>();
             result.push_str("..");
             result
@@ -440,49 +483,20 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
+                    self.layout
                 );
             }
             Colorname::Black => {
-                print_item!(
-                    color::Fg(color::Black),
-                    name,
-                    time,
-                    selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
-                );
+                print_item!(color::Fg(color::Black), name, time, selected, self.layout);
             }
             Colorname::Blue => {
-                print_item!(
-                    color::Fg(color::Blue),
-                    name,
-                    time,
-                    selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
-                );
+                print_item!(color::Fg(color::Blue), name, time, selected, self.layout);
             }
             Colorname::Cyan => {
-                print_item!(
-                    color::Fg(color::Cyan),
-                    name,
-                    time,
-                    selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
-                );
+                print_item!(color::Fg(color::Cyan), name, time, selected, self.layout);
             }
             Colorname::Green => {
-                print_item!(
-                    color::Fg(color::Green),
-                    name,
-                    time,
-                    selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
-                );
+                print_item!(color::Fg(color::Green), name, time, selected, self.layout);
             }
             Colorname::LightBlack => {
                 print_item!(
@@ -490,8 +504,7 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
+                    self.layout
                 );
             }
             Colorname::LightBlue => {
@@ -500,8 +513,7 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
+                    self.layout
                 );
             }
             Colorname::LightCyan => {
@@ -510,8 +522,7 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
+                    self.layout
                 );
             }
             Colorname::LightGreen => {
@@ -520,8 +531,7 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
+                    self.layout
                 );
             }
             Colorname::LightMagenta => {
@@ -530,8 +540,7 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
+                    self.layout
                 );
             }
             Colorname::LightRed => {
@@ -540,8 +549,7 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
+                    self.layout
                 );
             }
             Colorname::LightWhite => {
@@ -550,8 +558,7 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
+                    self.layout
                 );
             }
             Colorname::LightYellow => {
@@ -560,29 +567,14 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
+                    self.layout
                 );
             }
             Colorname::Magenta => {
-                print_item!(
-                    color::Fg(color::Magenta),
-                    name,
-                    time,
-                    selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
-                );
+                print_item!(color::Fg(color::Magenta), name, time, selected, self.layout);
             }
             Colorname::Red => {
-                print_item!(
-                    color::Fg(color::Red),
-                    name,
-                    time,
-                    selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
-                );
+                print_item!(color::Fg(color::Red), name, time, selected, self.layout);
             }
             Colorname::Rgb(x, y, z) => {
                 print_item!(
@@ -590,29 +582,14 @@ impl State {
                     name,
                     time,
                     selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
+                    self.layout
                 );
             }
             Colorname::White => {
-                print_item!(
-                    color::Fg(color::White),
-                    name,
-                    time,
-                    selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
-                );
+                print_item!(color::Fg(color::White), name, time, selected, self.layout);
             }
             Colorname::Yellow => {
-                print_item!(
-                    color::Fg(color::Yellow),
-                    name,
-                    time,
-                    selected,
-                    self.layout.time_start_pos,
-                    self.layout.terminal_column
-                );
+                print_item!(color::Fg(color::Yellow), name, time, selected, self.layout);
             }
         }
     }
@@ -696,8 +673,11 @@ impl State {
                     );
                 }
             }
-            if std::env::var("RUST_LOG") == Ok("debug".to_string()) {
-                print!(" index: {} skip: {}", nums.index, nums.skip);
+            if self.rust_log == Some("debug".to_string()) {
+                print!(
+                    " index:{} skip:{} max:{} column:{}",
+                    nums.index, nums.skip, self.layout.name_max_len, self.layout.terminal_column
+                );
             }
         }
         print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
@@ -716,53 +696,74 @@ impl State {
 
 fn make_item(entry: fs::DirEntry) -> ItemInfo {
     let path = entry.path();
-    let metadata = &fs::symlink_metadata(&path);
-
-    let time = match metadata {
-        Ok(metadata) => {
-            let sometime = metadata.modified().unwrap();
-            let chrono_time: DateTime<Local> = DateTime::from(sometime);
-            Some(chrono_time.to_rfc3339_opts(SecondsFormat::Secs, false))
-        }
-        Err(_) => None,
-    };
-
-    let filetype = match metadata {
-        Ok(metadata) => {
-            let file_type = metadata.file_type();
-            if file_type.is_dir() {
-                FileType::Directory
-            } else if file_type.is_file() {
-                FileType::File
-            } else if file_type.is_symlink() {
-                FileType::Symlink
-            } else {
-                FileType::File
-            }
-        }
-        Err(_) => FileType::File,
-    };
+    let metadata = fs::symlink_metadata(&path);
 
     let name = entry
         .file_name()
         .into_string()
         .unwrap_or_else(|_| "Invalid unicode name".to_string());
 
-    let size = match metadata {
-        Ok(metadata) => metadata.len(),
-        Err(_) => 0,
-    };
-
     let ext = path.extension().map(|s| s.to_os_string());
 
-    ItemInfo {
-        file_type: filetype,
-        file_name: name,
-        file_path: path,
-        file_size: size,
-        file_ext: ext,
-        modified: time,
-        selected: false,
+    match metadata {
+        Ok(metadata) => {
+            let time = {
+                let sometime = metadata.modified().unwrap();
+                let chrono_time: DateTime<Local> = DateTime::from(sometime);
+                Some(chrono_time.to_rfc3339_opts(SecondsFormat::Secs, false))
+            };
+
+            let filetype = {
+                let file_type = metadata.file_type();
+                if file_type.is_dir() {
+                    FileType::Directory
+                } else if file_type.is_file() {
+                    FileType::File
+                } else if file_type.is_symlink() {
+                    FileType::Symlink
+                } else {
+                    FileType::File
+                }
+            };
+
+            let sym_dir_path = {
+                if filetype == FileType::Symlink {
+                    if let Ok(sym_meta) = fs::metadata(&path) {
+                        if sym_meta.is_dir() {
+                            fs::canonicalize(path.clone()).ok()
+                        } else {
+                            None
+                        }
+                    } else {
+                        None
+                    }
+                } else {
+                    None
+                }
+            };
+
+            let size = metadata.len();
+            ItemInfo {
+                file_type: filetype,
+                file_name: name,
+                file_path: path,
+                symlink_dir_path: sym_dir_path,
+                file_size: size,
+                file_ext: ext,
+                modified: time,
+                selected: false,
+            }
+        }
+        Err(_) => ItemInfo {
+            file_type: FileType::File,
+            file_name: name,
+            file_path: path,
+            symlink_dir_path: None,
+            file_size: 0,
+            file_ext: ext,
+            modified: None,
+            selected: false,
+        },
     }
 }
 
